import django.conf.settings
from django_kepi.cache_model import Cache
from django_kepi import find

class _Thingy(object):

    @property
    def f_type(self):
        raise NotImplemented()

    def __init__(self, url):
        self.url = url
        self.is_local = url.startswith(settings.KEPI['LOCAL_OBJECT_PREFIX'])

        if self.is_local:
            self.referent = find(url,
                    f_type = self.f_type)
        else:
            self.referent = Cache.objects.get(url=url,
                    f_type = self.f_type)

        if self.referent is None:
            raise ValueError(url+' is not a known '+self.f_type)

    def __getitem__(self, key):
        return self.referent[key]

    def __str__(self):
        return '[{}={}]'.format(
                self.f_type,
                self.url,
                )

class Person(_Thingy):

    @property
    def f_type(self):
        return 'Person'

__all__ = [
        'Person',
        ]
